<?php

namespace Drupal\switch_theme\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;
use Drupal\switch_theme\SwitchthemeManager;

/**
 * Configure switch theme edit form
 */
class EditForm extends FormBase {
    /**
     * The database connection.
     *
     * @var \Drupal\Core\Database\Connection
     */
    protected $connection;

    /**
     * The switch theme manager.
     *
     * @var \Drupal\switch_theme\SwitchthemeManager
     */
    protected $switchemeManager;

    /**
     * Id
     *
     * @var mixed
     */
    protected $id;

    /**
     * Construct
     *
     * @param \Drupal\Core\Database\Connection       $connection       The database connection
     * @param \Drupal\switch_theme\SwitchthemeManager $switchemeManager The switch theme manager
     * 
     * @return void
     */
    public function __construct(Connection $connection, SwitchthemeManager $switchemeManager) {
        $this->connection = $connection;
        $this->switchemeManager = $switchemeManager;
    }
 
    /**
     * create
     *
     * @param ContainerInterface $container
     */
    public static function create(ContainerInterface $container) {
        return new static(
            $container->get('database'),
            $container->get('switch_theme.manager'),
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'switch_theme';
    }

    /**
     * Validate submitted form
     * 
     * @param array              $form
     * @param FormStateInterface $form_state
     *
     * @return void
    */
    public function validateForm(array &$form, FormStateInterface $form_state) {
        $roles = Role::loadMultiple();
        $themes = \Drupal::service('theme_handler')->listInfo();
        
        if (strlen($form_state->getValue('switch_theme_name')) == 0) {
            $form_state->setErrorByName('switch_theme_name', $this->t('You must specify a name.'));
        }

        $intersect = array_intersect($form_state->getValue('switch_theme_roles'), array_keys($roles));

        if (!$intersect) {
            $form_state->setErrorByName('switch_theme_roles', $this->t('The role does not exist.'));
        }

        if (strlen($form_state->getValue('switch_theme_pattern')) == 0) {
            $form_state->setErrorByName('switch_theme_pattern', $this->t('You must specify a pattern.'));
        }

        if (!in_array($form_state->getValue('switch_theme_theme'), array_keys($themes))) {
            $form_state->setErrorByName('switch_theme_theme', $this->t('The theme does not exist.'));
        }
    }
 
    /**
     * Build Switchtheme form
     * 
     * @param array              $form
     * @param FormStateInterface $form_state
     * @param int|null           $id
     *
     * @return array
     */
    public function buildForm(array $form, FormStateInterface $form_state, $id = null) {
        $this->id = $id;
        $editedItem = $this->switchemeManager->getItem($id);
        $selectedRoles = $this->switchemeManager->getSelectedRoles($id);

        $roles = Role::loadMultiple();
        $themes = \Drupal::service('theme_handler')->listInfo();
        
        $form['switch_theme_name'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Name'),
            '#default_value' => ($editedItem) ? $editedItem->name : "",
            '#description' => $this->t('Define switch theme name'),
            '#required' => true
        ];

        $options = [];
        foreach ($roles as $role) {
            $options[$role->id()] = $role->label();
        }
        
        $form['switch_theme_roles'] = [
            '#type' => 'checkboxes',
            '#title' => $this->t('Role(s)'),
            '#options' => $options,
            '#description' => $this->t('Which role(s) is concernerd ?'),
            '#default_value' => ($selectedRoles) ? $selectedRoles : [],
            '#required' => true
        ];

        $form['switch_theme_pattern'] = [
            '#type' => 'textfield',
            '#title' => 'URI Pattern',
            '#default_value' => ($editedItem) ? $editedItem->pattern : "",
            '#description' => $this->t('Define URI pattern'),
            '#required' => true
        ];

        $options = [];
        foreach ($themes as $id => $theme) {
            $options[$id] = $theme->getName();
        }
        
        $form['switch_theme_theme'] = [
            '#type' => 'radios',
            '#title' => $this->t('Theme'),
            '#options' => $options,
            '#description' => $this->t('Which theme to used ?'),
            '#default_value' => ($editedItem) ? $editedItem->theme_code : "",
            '#required' => true
        ];

        $form['submit'] = [
            '#type'             => 'submit',
            '#title'            => $this->t('Save'),
            '#default_value'    => "Save",
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state) {

        $name = $form_state->getValue('switch_theme_name');
        $roles = $form_state->getValue('switch_theme_roles');
        $pattern = $form_state->getValue('switch_theme_pattern');
        $theme = $form_state->getValue('switch_theme_theme');
        $id = $this->id;
       
        if ($id > 0) {
            $this->connection->update('switch_theme')
            ->fields(['name' => $name])
            ->condition('id', $id, "=")
            ->execute();
        } else {
            $id = $this->connection->insert('switch_theme')
            ->fields(['name' => $name])
            ->execute();
        }

        $this->connection->delete('switch_theme_role')
        ->condition('switch_theme_id', $id, '=')
        ->execute();

        $this->connection->delete('switch_theme_pattern')
        ->condition('switch_theme_id', $id, '=')
        ->execute();

        $this->connection->delete('switch_theme_theme')
        ->condition('switch_theme_id', $id, '=')
        ->execute();

        $query = $this->connection->insert('switch_theme_role')->fields(['switch_theme_id', 'role_code']);
        foreach ($roles as $role) {
            $query->values([$id, $role]);
        }
        $query->execute();  

        $this->connection->insert('switch_theme_pattern')
        ->fields(['switch_theme_id' => $id, 'pattern' => $pattern])
        ->execute();

        $this->connection->insert('switch_theme_theme')
        ->fields(['switch_theme_id' => $id, 'theme_code' => $theme])
        ->execute();
        
        $this->messenger()->addMessage($this->t('The Switch theme has been saved'));

        $response = Url::fromRoute('switch_theme.overview');
        $form_state->setRedirectUrl($response);
    } 
}
